import serial
import time
import re
import json

class communicate:
    cmd_list = []

    def __init__(self, port):
        self._port = port

    def _setcmd(self, cmd, end='\r\n'):
        end = '\r\n'

        return (cmd + end)
    def _readtill(self,till="OK"):
        rcv = self._port.read(14816)
        rcvd = rcv.decode('utf8', errors='ignore')
        while "OK" not in rcvd: 
            rcvd += rcv.decode('utf8', errors='ignore')
            rcv = self._port.read(14816)
        return rcvd
    def _ATcmd(self):
        self._port.write(self._setcmd("AT").encode())
        rcv = self._port.read(14816)

    def _send_cmd(self, cmd, t=1, bytes=14816, return_data=False, 
                printio=False, get_decode_data=False,read=True):
        cmd = self._setcmd(cmd)
        self._port.write(cmd.encode('utf8', errors='ignore'))
        if read:
            time.sleep(t)
            if not get_decode_data:
                rcv = self._port.read(bytes)
            else:
                rcv = None

            if printio:
                print(rcv.decode('utf8', errors='ignore'))
            
            if return_data:
                return rcv

    def _read_sent_data(self, numberOfBytes):
        rcv = self._port.read(numberOfBytes)

        return rcv

    def _bearer(self,APN):
        self._ATcmd()
        cmd = "AT +SAPBR=0,1"
        self._send_cmd(cmd)
        self._ATcmd()
        cmd = 'AT + SAPBR=3,1,"CONTYPE","GPRS"'
        self._send_cmd(cmd)
        cmd = 'AT + SAPBR=3,1,"APN","{}"'.format(APN)
        self._send_cmd(cmd)
        cmd = "AT + SAPBR=1,1"
        self._send_cmd(cmd)
        cmd = "AT + SAPBR=2,1"
        data = self._send_cmd(cmd,return_data=True)
        try :
           IP = data
        except:
            IP = None
        return IP

    def _getdata(self, data_to_decode=[], string_to_decode=None, till=b'\n', count=2, counter=0):

        rcv = self._port.read(1)

        if rcv == till:
            counter += 1
            if counter == count:
                data_to_decode.append(rcv)
                return b"".join(data_to_decode)
            else:
                data_to_decode.append(rcv)
                return self._getdata(data_to_decode, string_to_decode, till, count, counter)
        else:
            data_to_decode.append(rcv)
            return self._getdata(data_to_decode, string_to_decode, till, count, counter)



import queue
from . import stack
import time


class Parser:

    def __init__(self, text, *args, **kwargs):
        self.tokens = []
        self._parser = self.Parser(text)
        self._bytesparser = self.BytesParser(text)
        self.processText = "".join(text.decode('utf8', errors='ignore').split()) + "$"
        self.stack = stack.Stack()
        self.queue = queue.Queue()
        self.separator = ["+", ":", " ", ",", "$"]

    @property
    def parser(self):
        return self._parser

    @property
    def bytesparser(self):
        return self._bytesparser

    def tokenizer(self):
        for letter in self.processText:
            notin = False
            token = []
            if letter in self.separator:
                notin = True
        #         print(i ,x,notin)
                self.stack.push(letter)
                while not self.queue.empty():
                    token.append(self.queue.get())
                tokenString = "".join(token)
                if tokenString:
                    self.tokens.append("".join(token))
            if not notin:
                self.queue.put(letter)
        return self.tokens

    def Parser(self, text):
        text = text.replace(b"\r", b"")
        text = "".join(text.decode('utf8', errors='ignore').split("\n")[2:])
        text = "".join(text.split("OK")[0])
        return(text)

    def BytesParser(self, text):
        text = text.replace(b"\r", b"")
        text = b"".join(text.split(b"\n")[2:])
        text = text.split(b"OK")[0]
        return text



# from ATRequests import requests
from usim800.Sms import sms
from usim800.Communicate import  communicate
from usim800.Request import request
from usim800.Info import info
import serial

class sim800(communicate):
    TIMMEOUT = 1

    def __init__(self, baudrate, path):
        self.port = serial.Serial(path, baudrate, timeout=sim800.TIMMEOUT)
        super().__init__(self.port)
        self.requests = request(self.port)
        self.info = info(self.port)
        self.sms = sms(self.port)




from json import JSONDecoder


class ATJSONObjectParser:
    def __init__(self, text, *args, **kwargs):
        self.text = text.decode('utf8', errors='ignore')
        self._JSONObjectPresent = False
        self._JSONObject = []
        self.JSONObjectExreact(self.text)
        # self.get_data(text)

    @property
    def JSONObjectPresent(self):
        return self._JSONObjectPresent

    @property
    def JSONObject(self):
        return self._JSONObject

    def JSONObjectExreact(self, text):
        for object in self._extract_json_objects(text):
            self._JSONObject.append(object)
        if self._JSONObject:
            self._JSONObjectPresent = True

    def _extract_json_objects(self, text, decoder=JSONDecoder()):
        """Find JSON objects in text, and yield the decoded JSON data """

        processText = "".join(text.split())
        pos = 0
        while True:
            match_c = processText.find('{', pos)
            match_s = processText.find('[', pos)
            dif = abs(match_c-match_s)
            if dif == 1:
                match = text.find('[', pos)
            else:
                match = text.find('{', pos)
            if match == -1:
                break
            try:
                result, index = decoder.raw_decode(text[match:])
                yield result
                pos = match + index
            except ValueError:
                pos = match + 1




from usim800.Parser.ATParser import Parser
import time
from usim800.Communicate import communicate
from usim800.Parser import JsonParser


class info(communicate):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._IMEI = None
        self._ModuleVersion = None
        self._RSSI = None
        self._simoprator = None
        self._simostats = None
        self._APN = None
        self._Latitude , self._Longitude = None,None

    @property
    def APN(self):
        return self._APN

    @property
    def Location(self):
        return self._Latitude , self._Longitude


    @APN.setter
    def APN(self,val):
        self._APN = val

    @property
    def IMEI(self):
        return self._IMEI

    @property
    def simoprator(self):
        return self._simoprator

    @property
    def ModuleVersion(self):
        return self._ModuleVersion

    def getoprator(self):
        # https://stackoverflow.com/questions/39930218/sim800-gsm-module-returns-0-on-atcops
        cmd = "AT+CSPN?"
        data = self._send_cmd(cmd, return_data=True)
        try:
            simoprator = data.decode().split(":")[1].split(",")[
                0].replace('"', "")
        except:
            simoprator = None
        self._simoprator = simoprator
        print("simoprator ->", self._simoprator)
        return self._simoprator

    def getModuleVersion(self):
        """
        Get the module firmware version.
        """
        cmd = "AT+CGMR"
        data = self._send_cmd(cmd, return_data=True)
        # print(data)
        try:
            moduleVersion = data.decode().split()[1].split(":")[1].split()[0]
        except:
            moduleVersion = None
        self._ModuleVersion = moduleVersion
        print("ModuleVersion ->", self._ModuleVersion)

    def getIMEI(self):
        """
        Get the IMEI number of the module
        """
        cmd = "AT+GSN"
        data = self._send_cmd(cmd, return_data=True)
        # print(data.decode().split())
        try:
            IMEI = data.decode().split()[1]
        except:
            IMEI = None
        self._IMEI = IMEI
        print("IMEI->", self._IMEI)

    def checkSim(self):
        cmd = "AT+CMEE=2"  # enable the extended error codes to get a verbose format
        self._send_cmd(cmd)
        cmd = "AT+cpin?"
        data = self._send_cmd(cmd, return_data=True,t=2)
       
        try:
            simostats = data.decode().split(":")[1].split()[0]
        except:
            simostats = None
        self._simostats = simostats
        print("_simostats->", self._simostats)

    def getRSSI(self):
        """
        Get the current signal strength in 'bars'
        """
        cmd = "AT+CSQ"
        data = self._send_cmd(cmd, return_data=True)    
        try:
            RSSI = data.decode().split(":")[1].split()[0]
        except:
            RSSI = None
        self._RSSI = RSSI
        print("RSSI->", self._RSSI)
    def getLoctions(self):
        self._bearer(self._APN)
        cmd = "AT+CIPGSMLOC=1,1"
        data = self._send_cmd(cmd, return_data=True)   
        cmd = "AT+CIPGSMLOC=2,1"
        
        data = self._send_cmd(cmd, return_data=True)   
        try:
            
            self._Latitude , self._Longitude = data.decode().split()[1].split(",")[1] ,data.decode().split()[1].split(",")[2]
        except:
            self._Latitude , self._Longitude = None,None

        cmd = "AT +SAPBR=0,1"
        self._send_cmd(cmd)
        print(self._Latitude , self._Longitude )
    def getCBC(self):
        "battery info"
        cmd = "AT+CBC"
        data = self._send_cmd(cmd, return_data=True)  
        battery , voltage =  data.decode().split()[2].split(',')[1] ,data.decode().split()[2].split(',')[2]
        print(battery,int(voltage)/1000)
    
    def all(self):
        self.getCBC()
        self.getLoctions()
        self.getRSSI()
        self.checkSim()
        self.getIMEI()
        self.getModuleVersion()
        self.getoprator()
        




from usim800.Communicate import communicate

def _try_decode_utf16_encoded_string(s):
    # Check if the input string is a valid UTF-16 hex string. If it is, decode it. Otherwise, return it as-is. 
    #   is_utf16_encoded_string('60A87684') == '您的'
    #   is_utf16_encoded_string('hello') == 'hello'
    # Recolic K <root@recolic.net>
    prepared = s.strip().lower()
    if len(prepared) % 4 != 0:
        return s
    for c in prepared:
        if c not in '0123456789abcdef':
            return s
    # It is an utf16 encoded string. Let's decode it. 
    result_str = ''
    for i in range(len(prepared)//4):
        decoded_char = chr(int(prepared[i*4:(i+1)*4], 16))
        result_str += decoded_char
    return result_str



def _parse_cmgl_response(cmgl_response_str):
    # There is usually many lines in the response, and there may be duplicate entries and useless prefix&postfix. 
    # This function will trim these useless prefix/postfix, and parse the response. 
    # Recolic K <root@recolic.net>

    ENTRY_HEADLINE_PREFIX = '+CMGL: '
    # List entry headline format: ENTRY_HEADLINE_PREFIX ID ," READ_OR_UNREAD "," NUMBER ",""," DATETIME "
    result_dict = {}
    curr_entry = None
    for line in cmgl_response_str.replace('\r','\n').split('\n'):
        if line.startswith(ENTRY_HEADLINE_PREFIX):
            # New entry starts! End the previous entry and parse the new one. 
            if curr_entry is not None:
                result_dict[curr_entry[0]] = (curr_entry)
                curr_entry = None
            
            headline_fields = line.split(',"') # DATETIME string contains ','
            if len(headline_fields) != 5:
                print('Warning: Command AT+CMGL response is invalid. Expecting 5 fields in headline, but actually got {}. Skipping this bad response...'.format(len(headline_fields)))
                continue
            headline_fields = [f.strip('"') for f in headline_fields]
            msg_id = headline_fields[0][len(ENTRY_HEADLINE_PREFIX):]

            # Parsed. Now fill the curr_entry
            curr_entry = headline_fields
            curr_entry[0] = msg_id
            curr_entry.append('') # Body text
        else:
            # This line might be body of an entry, might be junk data. 
            if curr_entry is None:
                continue
            if line.strip() == '':
                continue
            if line.strip().lower() == 'ok':
                break
            # Not junk... Let's add text to curr_entry
            curr_entry[-1] += _try_decode_utf16_encoded_string(line) + '\n'

    # Exiting loop. Add the last entry to result! 
    if curr_entry is not None:
        result_dict[curr_entry[0]] = (curr_entry)
    return result_dict
 

class sms(communicate):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def send(self, number, sms):
        cmd = "AT"
        self._send_cmd(cmd)
        cmd = "AT+CMGF=1"
        # Sets the GSM Module in Text Mode
        self._send_cmd(cmd)
        cmd = 'AT+CMGS="{}"'.format(number)
        self._send_cmd(cmd)
        SMS = sms
        self._send_cmd(SMS,t=0.5)
        cmd = "\x1A"
        self._send_cmd(cmd,t=0.1)
        cmd = "AT +SAPBR=0,1"
        data = self._send_cmd(cmd,return_data=True,t=0.5)
        try:
            stats = (data.decode('utf8', errors='ignore').split()[-1])
            if "OK" in stats:
                stats = True
        except:
            stats = False
        return stats

    def readAll(self,index=None):
        # Sample Usage: 
        #   from usim800 import sim800
        #   gsm = sim800(baudrate=9600,path="/dev/ttyUSB0")
        #   res_dict = gsm.sms.readAll()
        #   for k in res_dict:
        #       print(res_dict[k])

        cmd = "AT"
        self._send_cmd(cmd)
        cmd = "AT+CMGF=1"
        # Sets the GSM Module in Text Mode
        self._send_cmd(cmd)
        cmd = 'AT+CMGL="ALL"'
        # Sets the GSM Module in Text Mode
        self._send_cmd(cmd,read=False)
        data = self._readtill("OK")
        return _parse_cmgl_response(data)

    def deleteAllReadMsg(self,index=None):
        # Delete all read message. Leave unread message and outgoing message untouched. 
        # You can provide the index of ANY existing message to make this function runs faster. 

        # API forces you to provide an index. So we must readAll()
        if index is None:
            msgs = self.readAll()
            if len(msgs) == 0:
                return
            index = list(msgs.keys())[0]
        if not isinstance(index, str):
            index = str(index)

        cmd = "AT"
        self._send_cmd(cmd)
        cmd = "AT+CMGD=" + index + ",1"
        # Sets the GSM Module in Text Mode
        self._send_cmd(cmd)





from  JsonParser import ATJSONObjectParser

data = """
kfjgifdjgiojidsjgiwdssge 23 32532 dgdg 123213 df df21e  
{ 
  "accounting"  : [   
                     { "firstName" : "John",  
                       "lastName"  : "Doe",
                       "age"       : 23 },

                     { "firstName" : "Mary",  
                       "lastName"  : "Smith",
                        "age"      : 32 }
                 ],                            
  "sales"      : [ 
                     { "firstName" : "Sally", 
                       "lastName"  : "Green",
                        "age"      : 27 },

                     { "firstName" : "Jim",   
                       "lastName"  : "Galley",
                       "age"       : 41 }
                 ] 
} kfjgifdjgiojidsjgiwdssge 23 32532 dgdg 123213 df df21e 
"""
data = "bhagyarsh"
jop = ATJSONObjectParser(data)
print(jop.JSONObjectPresent)
# print(type(jop.JSONObject[0]))


from usim800.Parser.ATParser import Parser
import time
from usim800.Communicate import communicate
from usim800.Parser import JsonParser


class request(communicate):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._status_code = None
        self._json = None
        self._text = None
        self._content = None
        self._url = None
        self._IP = None

    def init(self):
        self._status_code = None
        self._json = None
        self._text = None
        self._content = None
        self._url = None
        self._IP = None

    @property
    def text(self):
        return self._text

    @property
    def IP(self):
        return self._IP

    @property
    def APN(self):
        return self._APN

    @APN.setter
    def APN(self, APN):
        self._APN = APN

    @property
    def url(self):
        return self._url

    @property
    def content(self):
        return self._content

    @property
    def status_code(self):
        return self._status_code

    def json(self):
        return self._json

    def get(self, url, header=None):
        self.init()
        self._url = url
        self._IP = self._bearer(self._APN)

        cmd = "AT + HTTPINIT"
        self._send_cmd(cmd)
        cmd = 'AT + HTTPPARA="CID",1'
        self._send_cmd(cmd)

        cmd = 'AT + HTTPPARA="URL","{}"'.format(url)
        self._send_cmd(cmd)
        time.sleep(3)
        cmd = "AT +HTTPACTION=0"

        self._send_cmd(cmd)
        time.sleep(2)
        cmd = "AT +HTTPREAD"
        self._send_cmd(cmd, get_decode_data=True)
        data = self._getdata(
            data_to_decode=[], string_to_decode=None, till=b'\n', count=2, counter=0)
        tk = Parser(data)
        token = tk.tokenizer()
        self._content = tk.parser
        if (len(token) == 4):
            self._status_code = token[2]
            read_bytes = token[3]
            string = self._read_sent_data(int(read_bytes)+1000)
            tk = Parser(string)

            self._content = tk.bytesparser
            self._text = tk.parser
            jph = JsonParser.ATJSONObjectParser(string)
            self._json = jph.JSONObject
        cmd = "AT +SAPBR=0,1"
        self._send_cmd(cmd)

        return self._status_code

    def post(self, url, data, waittime=4000, bytes_data=None, headers=None):
        bytes_data = len(data) +100
        self.init()
        self._url = url
        self._IP = self._bearer(self._APN)
        cmd = "AT+HTTPINIT"
        self._send_cmd(cmd)
        cmd = 'AT+HTTPPARA="CID",1'
        self._send_cmd(cmd)
        cmd = 'AT+HTTPPARA="URL","{}"'.format(url)
        self._send_cmd(cmd)
        cmd = 'AT+HTTPPARA="CONTENT","application/json"'
        self._send_cmd(cmd)
        cmd = 'AT+HTTPDATA={},{}'.format(bytes_data, waittime)
        self._send_cmd(cmd)
        # self.post.write(data)
        self._send_cmd(data)
        time.sleep(4)
        cmd = 'AT+HTTPACTION=1'
        self._send_cmd(cmd)
        time.sleep(4)
        cmd = 'AT+HTTPREAD'
        self._send_cmd(cmd, get_decode_data=True)
        data = self._getdata(
            data_to_decode=[], string_to_decode=None, till=b'\n', count=2, counter=0)
        tk = Parser(data)
        self._content = tk.parser
        token = tk.tokenizer()
        if (len(token) == 4):
            self._status_code = token[2]
            read_bytes = token[3]
            string = self._read_sent_data(int(read_bytes)+1000)
            tk = Parser(string)
            self._content = tk.bytesparser
            self._text = tk.parser
            jph = JsonParser.ATJSONObjectParser(string)
            self._json = jph.JSONObject
        cmd = "AT +SAPBR=0,1"
        self._send_cmd(cmd)

        return self._status_code



class Stack:
    def __init__(self):
        self.items = []

    def isEmpty(self):
        return self.items == []

    def push(self, item):
        self.items.append(item)

    def pop(self):
        return self.items.pop()

    def peek(self):
        return self.items[len(self.items)-1]

    def show_items(self):
        return (self.items)
        
    def size(self):
        return len(self.items)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="usim800forutf8",
    version="0.1.3",
    author="fakdemir",
    author_email="rfrknakdemir@gmail.com",
    description="usim800 is a Python driver module for SIM800 GSM/GPRS (utf-8 support).",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/RFurkanakdemir/usim800forutf8",
    packages=setuptools.find_packages(exclude=("tests",)),
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    license = "MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    install_requires = ['pyserial']
)